let chaininfo = require("./chaininfo.json");
const { Api, JsonRpc, RpcError } = require("eosjs");
const { JsSignatureProvider } = require("eosjs/dist/eosjs-jssig");
const signatureProvider = new JsSignatureProvider([]);
const fetch = require("isomorphic-fetch");
const { TextEncoder, TextDecoder } = require("text-encoding");

const { createClient } = require("@liquidapps/dapp-client");

const Hashids = require("hashids/cjs");
const axios = require("axios").default;
const retry = require("async-retry");
const encoding = [
  "a",
  "b",
  "c",
  "d",
  "e",
  "f",
  "g",
  "h",
  "i",
  "j",
  "k",
  "l",
  "m",
  "n",
  "o",
];

Object.defineProperty(exports, "__esModule", { value: true });

exports.reguser = async (env, email, contact, otp, prvkey) => {
  try {
    const vacc = await this.converttovacc(email);
    let regobj = {
      vaccount: vacc,
      email: email,
      contact: contact,
      otp: otp,
      payload: {
        vaccount: vacc,
      },
    };
    let resp = await this.pushdappacc(
      env,
      chaininfo[env].contract,
      prvkey,
      "regboth",
      regobj
    );
    console.log(resp)
    if (resp.message) {
      return { accname: vacc };
    }
  } catch (err) {
    throw new Error(err);
  }
};

exports.recoveruser = async (env, email, otp, prvkey) => {
  try {
    const dappacc = await this.converttovacc(email);
    let recoverobj = {
      vaccount: dappacc,
      otp: otp,
      payload: {
        vaccount: dappacc,
      },
    };
    let resp = await this.pushdappacc(
      env,
      chaininfo[env].contract,
      prvkey,
      "confirmcode",
      recoverobj
    );
    if (resp.message) {
      return { accname: dappacc };
    }
  } catch (err) {
    throw new Error(err);
  }
};

async function gettrxbyid(env, trxid) {
  try {
    const dspEndpt = chaininfo[env].gettrx;
    const rpc = new JsonRpc(dspEndpt, { fetch });
    const api = new Api({
      rpc,
      signatureProvider,
      textDecoder: new TextDecoder(),
      textEncoder: new TextEncoder(),
    });
    let resp = await rpc.history_get_transaction(trxid);
    return resp;
  } catch (err) {
    const error = JSON.stringify(err);
    if (!(error.includes("trx not found") || error.includes("tx_not_found") || error.includes("not found"))) {
      throw new Error(err);
    }
    else {
      let resp = {}
      resp.error = error
      return resp
    }
  }
}

async function getCorrectTransactionId(env, transactionIds) {
  const requests = transactionIds.map((v) => gettrxbyid(env, v));
  const responses = await axios.all(requests);
  console.log("responses--", responses);
  let index = -1;
  for (let i = 0; i < responses.length; i++) {
    if (responses[i].error) {
      continue;
    } else {
      index = i;
      break;
    }
  }

  if (index >= 0) {
    return transactionIds[index];
  }

  return "";
}

async function pushTransaction(
  env,
  contract,
  pubkey,
  signature,
  payload,
  txIds
) {
  if (txIds.length) {
    console.log("here", txIds);
    const txId = await getCorrectTransactionId(env, txIds);
    if (txId.length > 0) {
      console.log("length", txId.length);
      return true;
    }
  }

  try {
    const service = await getservice(env, contract);
    const response = await service.post_liquid_account_transaction(
      contract,
      pubkey,
      signature,
      payload
    );
    console.log("--", response);
    const txId = response.result.transaction_id;
    txIds.push(txId);
    console.log("-id arr-", txIds);
    return pushTransaction(env, contract, pubkey, signature, payload, txIds);
  } catch (e) {
    console.log("err--", e);
    const error = JSON.stringify(e);
    console.log(error);
    if (
      error.includes("invalid nonce") ||
      error.includes("Provided public key already registered for this") ||
      error.includes("Call updateacckey to add keys") ||
      error.includes("duplicate transaction") ||
      error.includes("proposal already created for this user or company") ||  //for blokument only
      error.includes("user already signed for this proposal")
    ) {
      if (txIds.length) {
        return true
      }
      throw new Error(e);
    } else {
      throw new Error(e);
    }
  }
}

exports.signdappacc = async (env, contract, key, action, data) => {
  let resobj = {};
  const service = await getservice(env, contract);
  try {
    await retry(
      async (bail, num) => {
        resobj = {};
        if (num < 3) {
          console.log("Num", num);
          resobj = await service.sign_liquid_account_transaction(
            contract,
            key,
            action,
            data
          );
        }
      },
      {
        retries: 2,
      }
    );  
    return resobj;
  } catch (err) {
    console.log(err);
    return{}
  }
};

exports.postdappacc = async (env, contract, pubkey, signature, payload) => {
  try {
    let pushresult = await pushTransaction(
      env,
      contract,
      pubkey,
      signature,
      payload,
      []
    );
    if (pushresult) {
      return { message: "Successful" };
    }
  } catch (err) {
    throw new Error(err);
  }
};

exports.pushdappacc = async (env, contract, key, action, data) => {
  try {
     let resp = await this.signdappacc(env, contract, key, action, data);
     console.log('---', resp)
    if (resp.public_key) {
      let pushresult = await pushTransaction(
        env,
        contract,
        resp.public_key,
        resp.signature,
        resp.payload,
        []
      );
      console.log("pushtrx--", pushresult);
      if (pushresult) {
        return { message: "Successful" };
      }
    } else {
      throw new Error("Error in signing trx");
    }
  } catch (err) {
    throw new Error(err);
  }
};

async function getservice(env, contract) {
  let client = await createClient({
    network: "kylin",
    httpEndpoint: chaininfo[env].endpoint,
    fetch: fetch,
  });
  const service = await client.service("vaccounts", contract);
  return service;
}

exports.converttovacc = async (email) => {
  const hashids = new Hashids(email, 9);
  let hashsh = hashids.encode(999);
  let accname = "dap" + (await getaccname(hashsh.toLowerCase()));
  return accname;
};

async function getaccname(data) {
  let newdata = "";
  for (let i = 0; i < data.length; i += 1) {
    if (data.charAt(i) === 0 || data.charAt(i) > 5 || data.charAt(i) === "0") {
      newdata += encoding[data.charAt(i)];
    } else newdata += data.charAt(i);
  }
  return newdata;
}
