let dappaccount = {};

let { PrivateKey, PublicKey } = require("eosjs-ecc");
let helperaction = require("./common/helper");
const axios = require("axios").default;
let chaininfo = require("./common/chaininfo.json");

dappaccount.register_both = async (env, email, contact, otp) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    let prv_key = await PrivateKey.randomKey();
    prv_key = prv_key.toWif();
    const response_reg = await helperaction.reguser(
      env,
      email,
      contact,
      otp,
      prv_key
    );
    console.log("--res", response_reg);
    return {
      accname: response_reg.accname,
      key: prv_key,
    };
  } catch (err) {
    throw new Error(err);
  }
};

dappaccount.verifyemailreg = async (env, email) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    let resp = await axios.post(chaininfo[env].twofactregemail, {
      email: email,
    });
    return resp.data;
  } catch (err) {
    throw new Error(err.response.data.message);370812
  }
};


dappaccount.verifyemailreset = async (env, email) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    let dappacc = await helperaction.converttovacc(email.toLowerCase());
    let resp = await axios.post(chaininfo[env].twofactlogin, {
      vaccount: dappacc,
      email: email,
    });
    console.log(dappacc);
    return resp.data;
  } catch (err) {
    throw new Error(err.response.data.message);
  }
};

dappaccount.recover = async (env, email, auth_code) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    let prv_key = await PrivateKey.randomKey();
    prv_key = prv_key.toWif();
    const response_reg = await helperaction.recoveruser(
      env,
      email,
      auth_code,
      prv_key
    );
    console.log(response_reg);
    let resp = await axios.get(
      `${chaininfo[env].removeauthinfo}/${response_reg.accname}`
    );
    console.log("key-", prv_key);
    return {
      accname: response_reg.accname,
      key: prv_key,
      trxid: response_reg.trxid,
    };
  } catch (err) {
    throw new Error(err);
  }
};


dappaccount.signtrx = async (env, contract, key, action, data = {}) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    const resp = await helperaction.signdappacc(
      env,
      contract,
      key,
      action,
      data
    );
    if (resp.public_key) {
      return resp;
    } else {
      return new Error("Error in signing trx");
    }
  } catch (err) {
    throw new Error(err);
  }
};

dappaccount.posttrx = async (env, contract, pubkey, signature, payload) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    const resp = await helperaction.postdappacc(
      env,
      contract,
      pubkey,
      signature,
      payload
    );
    if (resp.message) {
      return resp;
    }
  } catch (err) {
    throw new Error(err);
  }
};

dappaccount.pushtrx = async (env, contract, key, action, data) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    const resp = await helperaction.pushdappacc(
      env,
      contract,
      key,
      action,
      data
    );
    if (resp.message) {
      return resp;
    }
  } catch (err) {
    throw new Error(err);
  }
};

dappaccount.converttodappacc = async (email) => {
  try {
    const resp = await helperaction.converttovacc(email);
    console.log(resp);
    if (resp) {
      return resp;
    }
  } catch (err) {
    throw new Error(err);
  }
};

module.exports = dappaccount;
